@@ -1,15 +1,19 @@
 # ==================================================================== #
 # TITLE                                                                #
-# Antimicrobial Resistance (AMR) Data Analysis for R                   #
+# AMR: An R Package for Working with Antimicrobial Resistance Data     #
 #                                                                      #
 # SOURCE                                                               #
 # https://github.com/msberends/AMR                                     #
 #                                                                      #
-# LICENCE                                                              #
-# (c) 2018-2022 Berends MS, Luz CF et al.                              #
-# Developed at the University of Groningen, the Netherlands, in        #
-# collaboration with non-profit organisations Certe Medical            #
-# Diagnostics & Advice, and University Medical Center Groningen.       # 
+# CITE AS                                                              #
+# Berends MS, Luz CF, Friedrich AW, Sinha BNM, Albers CJ, Glasner C    #
+# (2022). AMR: An R Package for Working with Antimicrobial Resistance  #
+# Data. Journal of Statistical Software, 104(3), 1-31.                 #
+# doi:10.18637/jss.v104.i03                                            #
+#                                                                      #
+# Developed at the University of Groningen and the University Medical  #
+# Center Groningen in The Netherlands, in collaboration with many      #
+# colleagues from around the world, see our website.                   #
 #                                                                      #
 # This R package is free software; you can freely use and distribute   #
 # it for both personal and commercial purposes under the terms of the  #
@@ -24,56 +28,82 @@
 # ==================================================================== #
 
 # these are allowed MIC values and will become [factor] levels
-ops <- c("<", "<=", "", ">=", ">")
-valid_mic_levels <- c(c(t(vapply(FUN.VALUE = character(9), ops,
-                                 function(x) paste0(x, "0.00", 1:9)))),
-                      unique(c(t(vapply(FUN.VALUE = character(104), ops,
-                                        function(x) paste0(x, sort(as.double(paste0("0.0", 
-                                                                                    sort(c(1:99, 125, 128, 256, 512, 625)))))))))),
-                      unique(c(t(vapply(FUN.VALUE = character(103), ops,
-                                        function(x) paste0(x, sort(as.double(paste0("0.", 
-                                                                                    c(1:99, 125, 128, 256, 512))))))))),
-                      c(t(vapply(FUN.VALUE = character(10), ops,
-                                 function(x) paste0(x, sort(c(1:9, 1.5)))))),
-                      c(t(vapply(FUN.VALUE = character(45), ops,
-                                 function(x) paste0(x, c(10:98)[9:98 %% 2 == TRUE])))),
-                      c(t(vapply(FUN.VALUE = character(17), ops,
-                                 function(x) paste0(x, sort(c(2 ^ c(7:11), 192, 80 * c(2:12))))))))
+operators <- c("<", "<=", "", ">=", ">")
+valid_mic_levels <- c(
+  c(t(vapply(
+    FUN.VALUE = character(6), operators,
+    function(x) paste0(x, "0.000", c(1:4, 6, 8))
+  ))),
+  c(t(vapply(
+    FUN.VALUE = character(90), operators,
+    function(x) paste0(x, "0.00", c(1:9, 11:19, 21:29, 31:39, 41:49, 51:59, 61:69, 71:79, 81:89, 91:99))
+  ))),
+  unique(c(t(vapply(
+    FUN.VALUE = character(106), operators,
+    function(x) {
+      paste0(x, sort(as.double(paste0(
+        "0.0",
+        sort(c(1:99, 125, 128, 156, 165, 256, 512, 625))
+      ))))
+    }
+  )))),
+  unique(c(t(vapply(
+    FUN.VALUE = character(103), operators,
+    function(x) {
+      paste0(x, sort(as.double(paste0(
+        "0.",
+        c(1:99, 125, 128, 256, 512)
+      ))))
+    }
+  )))),
+  c(t(vapply(
+    FUN.VALUE = character(10), operators,
+    function(x) paste0(x, sort(c(1:9, 1.5)))
+  ))),
+  c(t(vapply(
+    FUN.VALUE = character(45), operators,
+    function(x) paste0(x, c(10:98)[9:98 %% 2 == TRUE])
+  ))),
+  c(t(vapply(
+    FUN.VALUE = character(17), operators,
+    function(x) paste0(x, sort(c(2^c(7:11), 192, 80 * c(2:12))))
+  )))
+)
 
 #' Transform Input to Minimum Inhibitory Concentrations (MIC)
 #'
 #' This transforms vectors to a new class [`mic`], which treats the input as decimal numbers, while maintaining operators (such as ">=") and only allowing valid MIC values known to the field of (medical) microbiology.
-#' @inheritSection lifecycle Stable Lifecycle
 #' @rdname as.mic
 #' @param x a [character] or [numeric] vector
 #' @param na.rm a [logical] indicating whether missing values should be removed
-#' @details To interpret MIC values as RSI values, use [as.rsi()] on MIC values. It supports guidelines from EUCAST and CLSI.
-#' 
+#' @param ... arguments passed on to methods
+#' @details To interpret MIC values as SIR values, use [as.sir()] on MIC values. It supports guidelines from EUCAST (`r min(as.integer(gsub("[^0-9]", "", subset(clinical_breakpoints, guideline %like% "EUCAST")$guideline)))`-`r max(as.integer(gsub("[^0-9]", "", subset(clinical_breakpoints, guideline %like% "EUCAST")$guideline)))`) and CLSI (`r min(as.integer(gsub("[^0-9]", "", subset(clinical_breakpoints, guideline %like% "CLSI")$guideline)))`-`r max(as.integer(gsub("[^0-9]", "", subset(clinical_breakpoints, guideline %like% "CLSI")$guideline)))`).
+#'
 #' This class for MIC values is a quite a special data type: formally it is an ordered [factor] with valid MIC values as [factor] levels (to make sure only valid MIC values are retained), but for any mathematical operation it acts as decimal numbers:
-#' 
+#'
 #' ```
 #' x <- random_mic(10)
 #' x
-#' #> Class <mic>
+#' #> Class 'mic'
 #' #>  [1] 16     1      8      8      64     >=128  0.0625 32     32     16
-#' 
+#'
 #' is.factor(x)
 #' #> [1] TRUE
-#' 
+#'
 #' x[1] * 2
 #' #> [1] 32
-#' 
+#'
 #' median(x)
 #' #> [1] 26
 #' ```
-#' 
+#'
 #' This makes it possible to maintain operators that often come with MIC values, such ">=" and "<=", even when filtering using [numeric] values in data analysis, e.g.:
-#' 
+#'
 #' ```
 #' x[x > 4]
-#' #> Class <mic>
+#' #> Class 'mic'
 #' #> [1] 16    8     8     64    >=128 32    32    16
-#' 
+#'
 #' df <- data.frame(x, hospital = "A")
 #' subset(df, x > 4) # or with dplyr: df %>% filter(x > 4)
 #' #>        x hospital
@@ -84,51 +114,74 @@ valid_mic_levels <- c(c(t(vapply(FUN.VALUE = character(9), ops,
 #' #> 9     32        A
 #' #> 10    16        A
 #' ```
-#' 
+#'
 #' The following [generic functions][groupGeneric()] are implemented for the MIC class: `!`, `!=`, `%%`, `%/%`, `&`, `*`, `+`, `-`, `/`, `<`, `<=`, `==`, `>`, `>=`, `^`, `|`, [abs()], [acos()], [acosh()], [all()], [any()], [asin()], [asinh()], [atan()], [atanh()], [ceiling()], [cos()], [cosh()], [cospi()], [cummax()], [cummin()], [cumprod()], [cumsum()], [digamma()], [exp()], [expm1()], [floor()], [gamma()], [lgamma()], [log()], [log1p()], [log2()], [log10()], [max()], [mean()], [min()], [prod()], [range()], [round()], [sign()], [signif()], [sin()], [sinh()], [sinpi()], [sqrt()], [sum()], [tan()], [tanh()], [tanpi()], [trigamma()] and [trunc()]. Some functions of the `stats` package are also implemented: [median()], [quantile()], [mad()], [IQR()], [fivenum()]. Also, [boxplot.stats()] is supported. Since [sd()] and [var()] are non-generic functions, these could not be extended. Use [mad()] as an alternative, or use e.g. `sd(as.numeric(x))` where `x` is your vector of MIC values.
+#'
+#' Using [as.double()] or [as.numeric()] on MIC values will remove the operators and return a numeric vector. Do **not** use [as.integer()] on MIC values as by the \R convention on [factor]s, it will return the index of the factor levels (which is often useless for regular users).
+#'
+#' Use [droplevels()] to drop unused levels. At default, it will return a plain factor. Use `droplevels(..., as.mic = TRUE)` to maintain the `mic` class.
 #' @return Ordered [factor] with additional class [`mic`], that in mathematical operations acts as decimal numbers. Bare in mind that the outcome of any mathematical operation on MICs will return a [numeric] value.
 #' @aliases mic
 #' @export
-#' @seealso [as.rsi()]
-#' @inheritSection AMR Read more on Our Website!
+#' @seealso [as.sir()]
 #' @examples
 #' mic_data <- as.mic(c(">=32", "1.0", "1", "1.00", 8, "<=0.128", "8", "16", "16"))
+#' mic_data
 #' is.mic(mic_data)
 #'
-#' # this can also coerce combined MIC/RSI values:
-#' as.mic("<=0.002; S") # will return <=0.002
-#' 
-#' # mathematical processing treats MICs as [numeric] values
+#' # this can also coerce combined MIC/SIR values:
+#' as.mic("<=0.002; S")
+#'
+#' # mathematical processing treats MICs as numeric values
 #' fivenum(mic_data)
 #' quantile(mic_data)
 #' all(mic_data < 512)
 #'
 #' # interpret MIC values
-#' as.rsi(x = as.mic(2),
-#'        mo = as.mo("S. pneumoniae"),
-#'        ab = "AMX",
-#'        guideline = "EUCAST")
-#' as.rsi(x = as.mic(4),
-#'        mo = as.mo("S. pneumoniae"),
-#'        ab = "AMX",
-#'        guideline = "EUCAST")
+#' as.sir(
+#'   x = as.mic(2),
+#'   mo = as.mo("Streptococcus pneumoniae"),
+#'   ab = "AMX",
+#'   guideline = "EUCAST"
+#' )
+#' as.sir(
+#'   x = as.mic(c(0.01, 2, 4, 8)),
+#'   mo = as.mo("Streptococcus pneumoniae"),
+#'   ab = "AMX",
+#'   guideline = "EUCAST"
+#' )
 #'
 #' # plot MIC values, see ?plot
 #' plot(mic_data)
 #' plot(mic_data, mo = "E. coli", ab = "cipro")
+#'
+#' if (require("ggplot2")) {
+#'   autoplot(mic_data, mo = "E. coli", ab = "cipro")
+#' }
+#' if (require("ggplot2")) {
+#'   autoplot(mic_data, mo = "E. coli", ab = "cipro", language = "nl") # Dutch
+#' }
+#' if (require("ggplot2")) {
+#'   autoplot(mic_data, mo = "E. coli", ab = "cipro", language = "uk") # Ukrainian
+#' }
 as.mic <- function(x, na.rm = FALSE) {
   meet_criteria(x, allow_class = c("mic", "character", "numeric", "integer", "factor"), allow_NA = TRUE)
   meet_criteria(na.rm, allow_class = "logical", has_length = 1)
-  
+
   if (is.mic(x)) {
     x
   } else {
-    x <- as.character(unlist(x))
-    if (na.rm == TRUE) {
+    if (is.numeric(x)) {
+      x <- format(x, scientific = FALSE)
+    } else {
+      x <- as.character(unlist(x))
+    }
+    if (isTRUE(na.rm)) {
       x <- x[!is.na(x)]
     }
+    x[trimws2(x) == ""] <- NA
     x.bak <- x
-    
+
     # comma to period
     x <- gsub(",", ".", x, fixed = TRUE)
     # transform Unicode for >= and <=
@@ -161,28 +214,35 @@ as.mic <- function(x, na.rm = FALSE) {
     # never end with dot
     x <- gsub("[.]$", "", x, perl = TRUE)
     # trim it
-    x <- trimws(x)
-    
+    x <- trimws2(x)
+
     ## previously unempty values now empty - should return a warning later on
     x[x.bak != "" & x == ""] <- "invalid"
-    
+
     na_before <- x[is.na(x) | x == ""] %pm>% length()
     x[!x %in% valid_mic_levels] <- NA
     na_after <- x[is.na(x) | x == ""] %pm>% length()
-    
+
     if (na_before != na_after) {
       list_missing <- x.bak[is.na(x) & !is.na(x.bak) & x.bak != ""] %pm>%
         unique() %pm>%
         sort() %pm>%
         vector_and(quotes = TRUE)
-      warning_("in `as.mic()`: ", na_after - na_before, " results truncated (",
-               round(((na_after - na_before) / length(x)) * 100),
-               "%) that were invalid MICs: ",
-               list_missing, call = FALSE)
+      cur_col <- get_current_column()
+      warning_("in `as.mic()`: ", na_after - na_before, " result",
+        ifelse(na_after - na_before > 1, "s", ""),
+        ifelse(is.null(cur_col), "", paste0(" in column '", cur_col, "'")),
+        " truncated (",
+        round(((na_after - na_before) / length(x)) * 100),
+        "%) that were invalid MICs: ",
+        list_missing,
+        call = FALSE
+      )
     }
-    
+
     set_clean_class(factor(x, levels = valid_mic_levels, ordered = TRUE),
-                    new_class =  c("mic", "ordered", "factor"))
+      new_class = c("mic", "ordered", "factor")
+    )
   }
 }
 
@@ -191,15 +251,18 @@ all_valid_mics <- function(x) {
     return(FALSE)
   }
   x_mic <- tryCatch(suppressWarnings(as.mic(x[!is.na(x)])),
-                    error = function(e) NA)
+    error = function(e) NA
+  )
   !any(is.na(x_mic)) && !all(is.na(x))
 }
 
 #' @rdname as.mic
-#' @details `NA_mic_` is a missing value of the new `<mic>` class.
+#' @details `NA_mic_` is a missing value of the new `mic` class, analogous to e.g. base \R's [`NA_character_`][base::NA].
+#' @format NULL
 #' @export
 NA_mic_ <- set_clean_class(factor(NA, levels = valid_mic_levels, ordered = TRUE),
-                           new_class =  c("mic", "ordered", "factor"))
+  new_class = c("mic", "ordered", "factor")
+)
 
 #' @rdname as.mic
 #' @export
@@ -214,13 +277,6 @@ as.double.mic <- function(x, ...) {
   as.double(gsub("[<=>]+", "", as.character(x), perl = TRUE))
 }
 
-#' @method as.integer mic
-#' @export
-#' @noRd
-as.integer.mic <- function(x, ...) {
-  as.integer(gsub("[<=>]+", "", as.character(x), perl = TRUE))
-}
-
 #' @method as.numeric mic
 #' @export
 #' @noRd
@@ -228,11 +284,12 @@ as.numeric.mic <- function(x, ...) {
   as.numeric(gsub("[<=>]+", "", as.character(x), perl = TRUE))
 }
 
+#' @rdname as.mic
 #' @method droplevels mic
+#' @param as.mic a [logical] to indicate whether the `mic` class should be kept - the default is `FALSE`
 #' @export
-#' @noRd
-droplevels.mic <- function(x, exclude = if (any(is.na(levels(x)))) NULL else NA, as.mic = TRUE, ...) {
-  x <- droplevels.factor(x, exclude = exclude, ...)
+droplevels.mic <- function(x, as.mic = FALSE, ...) {
+  x <- droplevels.factor(x, ...)
   if (as.mic == TRUE) {
     class(x) <- c("mic", "ordered", "factor")
   }
@@ -260,7 +317,11 @@ type_sum.mic <- function(x, ...) {
 #' @export
 #' @noRd
 print.mic <- function(x, ...) {
-  cat("Class <mic>\n")
+  cat("Class 'mic'",
+    ifelse(length(levels(x)) < length(valid_mic_levels), font_red(" with dropped levels"), ""),
+    "\n",
+    sep = ""
+  )
   print(as.character(x), quote = FALSE)
   att <- attributes(x)
   if ("na.action" %in% names(att)) {
@@ -366,12 +427,12 @@ hist.mic <- function(x, ...) {
 get_skimmers.mic <- function(column) {
   skimr::sfl(
     skim_type = "mic",
-    p0 = ~stats::quantile(., probs = 0, na.rm = TRUE, names = FALSE),
-    p25 = ~stats::quantile(., probs = 0.25, na.rm = TRUE, names = FALSE),
-    p50 = ~stats::quantile(., probs = 0.5, na.rm = TRUE, names = FALSE),
-    p75 = ~stats::quantile(., probs = 0.75, na.rm = TRUE, names = FALSE),
-    p100 = ~stats::quantile(., probs = 1, na.rm = TRUE, names = FALSE),
-    hist = ~skimr::inline_hist(log2(stats::na.omit(.)), 5)
+    p0 = ~ stats::quantile(., probs = 0, na.rm = TRUE, names = FALSE),
+    p25 = ~ stats::quantile(., probs = 0.25, na.rm = TRUE, names = FALSE),
+    p50 = ~ stats::quantile(., probs = 0.5, na.rm = TRUE, names = FALSE),
+    p75 = ~ stats::quantile(., probs = 0.75, na.rm = TRUE, names = FALSE),
+    p100 = ~ stats::quantile(., probs = 1, na.rm = TRUE, names = FALSE),
+    hist = ~ skimr::inline_hist(log2(stats::na.omit(.)), 5)
   )
 }
 
@@ -667,7 +728,7 @@ is_lower <- function(el) {
 #' @export
 #' @noRd
 `^.mic` <- function(e1, e2) {
-  as.double(e1) ^ as.double(e2)
+  as.double(e1)^as.double(e2)
 }
 
 #' @method %% mic
